@@ -30,21 +30,20 @@
 #ifdef _WIN32
 #include <direct.h>
 #endif /* _WIN32 */
-#include <stdio.h>
-#include <string>
-#include <cassert>
-#include <memory>
 #include <algorithm>
 #include <array>
-#include <wchar.h>
+#include <bit>
+#include <cassert>
+#include <format>
+#include <fstream>
+#include <filesystem>
+#include <iostream>
+#include <memory>
+#include <ranges>
+#include <string>
 #include <vector>
-#include <inttypes.h>
-#include <io.h>
-#include <stdio.h>
 #include <sal.h>
 
-/*don't convert this to constexpr because it cause the 
-compiler to complain with the printf statement.*/
 constexpr auto DLL_NAME = L"d3dcompiler_47.dll";
 
 #ifndef D3D_COMPILE_STANDARD_FILE_INCLUDE
