@@ -15,26 +15,21 @@ void CompilerParams::initializeDefines() {
     defines = std::make_shared<std::vector < CompilerDefine >> ();
 }
 
-void CompilerParams::add_define(const std::string& defineOption) {
+void CompilerParams::add_define(const std::string_view& defineOption) {
     CompilerDefine _def;
-    size_t delin_pos;
     _def.Definition = "1";
 
-    delin_pos = defineOption.find('=');
-    if (delin_pos == std::string::npos) {
+    if (size_t Dellin_pos = defineOption.find('='); Dellin_pos == std::string::npos) {
         _def.Name = defineOption;
     }
     else {
-        _def.Name = defineOption.substr(0, delin_pos);
-        _def.Definition = defineOption.substr(delin_pos + 1, defineOption.length() - 1);
-
+        _def.Name = defineOption.substr(0, Dellin_pos);
+        _def.Definition = defineOption.substr(Dellin_pos + 1, defineOption.length() - 1);
     }
-   
     
     defines->insert(defines->begin(), _def);
 }
 
-void CompilerParams::LoadSourceCode(FILE* f)
-{
-    readall(f, SourceCode);
-}
+void CompilerParams::LoadSourceCode(std::ifstream& f) {
+    readAll(f, SourceCode);
+}
\ No newline at end of file
